import uvicorn
from fastapi import FastAPI, File, UploadFile,HTTPException,WebSocket
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import JSONResponse,StreamingResponse
from typing import List
import cv2
import numpy as np
from tensorflow.keras.models import load_model
from paddy import read_vid
from tempfile import NamedTemporaryFile
import os
import pandas as pd
import numpy as np
from statsmodels.tsa.statespace.sarimax import SARIMAX
import pyrebase
from ultralytics import YOLO

app = FastAPI()
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_methods=["*"],
    allow_headers=["*"],
)

firebaseConfig = {
  "apiKey": "AIzaSyA-Ab3684hy2PrG1M5gPLShQG1pR9Mc4nE",
  "authDomain": "iotdb-87405.firebaseapp.com",
  "databaseURL": "https://iotdb-87405-default-rtdb.firebaseio.com",
  "projectId": "iotdb-87405",
  "storageBucket": "iotdb-87405.appspot.com",
  "messagingSenderId": "111379794358",
  "appId": "1:111379794358:web:d2320dc85fcea59d765917",
  "measurementId": "G-GWK6RNC7QY"
}

firebase = pyrebase.initialize_app(firebaseConfig)
db = firebase.database()

# Load the pre-trained model
loaded_model = load_model('weed_detection_model.h5', compile=False)

# Define the label mapping
labels = ["broadleaf", "grass", "soil", "soybean"]

# Function to preprocess and predict the image
def predict_image(image_path):
    try:
        image = cv2.imread(image_path)
        if image is None:
            return None
        resized_img = cv2.resize(image, (227, 227))
        resized_img = np.expand_dims(resized_img, axis=0)  # Add batch dimension
        predictions = loaded_model.predict(resized_img)
        predicted_class = np.argmax(predictions, axis=1)[0]
        return labels[predicted_class]
    except Exception as e:
        print(e)
        return None

@app.post("/predict/")
async def predict(file: UploadFile):
    try:
        # Save the uploaded file
        with open("uploaded_image.tif", "wb") as f:
            f.write(file.file.read())
        
        # Predict the uploaded image
        prediction = predict_image("uploaded_image.tif")

        if prediction is None:
            return JSONResponse(content={"error": "Invalid image file"}, status_code=400)

        return {"prediction": prediction}
    except Exception as e:
        return JSONResponse(content={"error": str(e)}, status_code=500)
    
@app.post("/video/detect-dense")
def detect_faces(file: UploadFile = File(...)):
    temp = NamedTemporaryFile(delete=False)
    try:
        try:
            contents = file.file.read()
            with temp as f:
                f.write(contents)
        except Exception:
            return {"message": "There was an error uploading the file"}
        finally:
            file.file.close()
        
        res = process_video(temp.name)  
    except Exception:
        return {"message": "There was an error processing the file"}
    finally:
        os.remove(temp.name)
    return res

def process_video(vid):
    print("process_video")
    data=read_vid(vid)
    print(data[0])
    return {"results":round(data[0])}

# @app.post("/getDevices")
# def getDevicesData():

device_data = {
    'D01': {'Moisture': '100', 'dht': '29'},
    'D02': {'Moisture': '50', 'dht': '29'},
    'D03': {'Moisture': '50', 'dht': '29'},
    'D04': {'Moisture': '50', 'dht': '29'},
    'D05': {'Moisture': '50', 'dht': '29'},
    'D06': {'Moisture': '50', 'dht': '29'}
}

@app.get("/device_data")
async def get_device_data():
    return {"results":device_data} 

def get_pred():
    df = pd.read_csv('DataTime-1.csv')
    # Set 'Date' as the index
    df['Date'] = pd.to_datetime(df['Date'], format='%Y-%m-%d')
    df.set_index('Date', inplace=True)

    # Train-test split
    train = df.iloc[:int(len(df) * 0.8)]
    test = df.iloc[int(len(df) * 0.8):]

    # Fit the SARIMAX model
    exog_features = train[['Avg Temp (°C)', 'Rainfall (mm)']]
    model = SARIMAX(train['Water Supply (ML)'], exog=exog_features, order=(1, 1, 1), seasonal_order=(1, 1, 1, 12))
    results = model.fit()

    # Make predictions on the test data
    exog_test = test[['Avg Temp (°C)', 'Rainfall (mm)']]
    predictions = results.get_forecast(steps=len(test), exog=exog_test)
    predicted_values = predictions.predicted_mean

    return predicted_values, test


@app.get("/predict_Water Supply/")
async def get_prediction(date: str, device: str):
    try:
        pred_df, test_data = get_pred()
        soil = db.child(device).child("Moisture").get().val()
        Avg_Temp = db.child(device).child("dht").get().val()
        
        if int(soil[0]) < 50:
            date = pd.to_datetime(date, format='%Y-%m-%d')
            exog_features = [date,soil,Avg_Temp]
            if date in pred_df.index:
                prediction = pred_df.loc[date]
            else:
                exog_features = test_data.loc[date, ['Avg Temp (°C)', 'Rainfall (mm)']]
                predictions_NEW, _ = get_pred()
                prediction = predictions_NEW.loc[date]

            return {"date": date.strftime('%Y-%m-%d'), "predicted Water Supply": round(prediction), "soil moisture": soil}
        else:
            return {"date": date, "soil moisture": soil}
    except KeyError:
        raise HTTPException(status_code=404, detail="Date not found in predictions")
    
URL = "http://192.168.43.182"
AWB = True

# cap = cv2.VideoCapture(URL + ":81/stream")
cap= cv2.VideoCapture(0)
model = YOLO('best (7).pt')

def generate_frames():
    while True:
        if cap.isOpened():
            ret, frame = cap.read()
            if ret:
                results = model(frame, stream=True)
                # Your processing code here
                # You can return the processed frame as bytes
                ret, buffer = cv2.imencode('.jpg', frame)
                if ret:
                    yield (b'--frame\r\n'
                           b'Content-Type: image/jpeg\r\n\r\n' + buffer.tobytes() + b'\r\n')
                # cv2.imshow("frame", frame)
                # # key = cv2.waitKey(1)
                # if cv2.waitKey(1) & 0xFF == ord("q"):
                #     break
@app.get("/video_stream")
async def video_stream():
    return StreamingResponse(generate_frames(), media_type="multipart/x-mixed-replace; boundary=frame")
    
if __name__ == "__main__":
    uvicorn.run("main:app", host="127.0.0.1",port=8000, log_level="info")
